==> nest-infrastructure-middleware-redis-3.5.3.0-20251216.104358-11-sources.jar: com/digiwin/dap/nest/infrastructure/middleware/redis/DwRedisLock.java <==
package com.digiwin.dap.nest.infrastructure.middleware.redis;

import com.digiwin.dap.nest.kernel.core.dapper.log.DwLog;
import com.digiwin.dap.nest.kernel.core.lock.distributed.DwDistributedLockAdapter;
import com.digiwin.dap.nest.kernel.core.lock.distributed.DwDistributedLockEntity;
import com.digiwin.dap.nest.kernel.core.lock.distributed.DwDistributedLockAspect;
import com.digiwin.dap.nest.kernel.core.util.DwStringUtil;
import com.digiwin.dap.nest.kernel.core.util.JaValidateUtil;
import com.digiwin.dap.nest.kernel.core.util.algorithm.id.JaUUID;
import com.digiwin.dap.nest.kernel.meta.adapter.JaI18nAdapter;
import com.digiwin.dap.nest.kernel.meta.function.JaFunctionTR;
import lombok.SneakyThrows;
import org.springframework.transaction.support.TransactionSynchronizationAdapter;
import org.springframework.transaction.support.TransactionSynchronizationManager;

import java.util.Set;

/**
 * redis 分布式锁
 *
 * @author chenjian
 * @since 2020年09月07日 10:00:27
 */
public class DwRedisLock {

    private DwRedisLock() {
    }


    /**
     * 使用demo 可参考{@link DwDistributedLockAspect}
     *
     * @author chenjian
     * @since 2020年10月13日 12:04:29
     */
    @SneakyThrows
    public static <T> T lock(DwDistributedLockEntity entity, JaFunctionTR<T> callable) {
        String lockKey = entity.getLockKey();
        String apiName = entity.getApiName();
        if (DwStringUtil.isEmpty(lockKey)) {
            DwLog.get().warn("lockKey is null,{}", apiName);
            return callable.apply();
        }
        // 判断是否可重入
        Set<String> reentrantLockKeySet = DwDistributedLockAdapter.getReentrantThreadLocal().getNotNull();
        if (reentrantLockKeySet.contains(lockKey)) {
            return callable.apply();
        }
        DwLog.biz("lockKey:{}", lockKey);
        // TODO 应对误删, 其实防不住, 待思考
        String requestId = lockKey + ":" + JaUUID.UUID19();
        boolean lock = DwDistributedLockAdapter.spin(entity, () -> DwRedis.setNx(lockKey, requestId, entity.getTimeout()));
        JaValidateUtil.isTrue(!lock, () -> JaI18nAdapter.getMessage(entity.getMessage()));
        try {
            reentrantLockKeySet.add(lockKey);
            return callable.apply();
        } finally {
            unlock(lockKey, requestId, entity.isTx());
        }
    }

    /**
     * 释放锁
     *
     * @author chenjian
     * @since 2020年08月11日 11:43:45
     */
    private static void unlock(String lockKey, String requestId, boolean tx) {
        if (TransactionSynchronizationManager.isSynchronizationActive() && tx) {
            try {
                TransactionSynchronizationManager.registerSynchronization(new TransactionSynchronizationAdapter() {
                    @Override
                    public void afterCompletion(int status) {
                        unlock(lockKey, requestId);
                    }
                });
            } catch (Throwable e) {
                DwLog.get().error("Transaction afterCompletion unLock lock failed", e);
                unlock(lockKey, requestId);
            }
        } else {
            unlock(lockKey, requestId);
        }
    }

    /**
     * TODO 暂用del
     * 是否要使用unlink
     * org.springframework.data.redis.core.RedisTemplate#unlink(java.lang.Object)
     */
    private static void unlock(String lockKey, String requestId) {
        try {
            String currentValue = DwRedis.getString(lockKey);
            if (!requestId.equals(currentValue)) {
                DwLog.get().warn("redis unlock warn {}:[currentValue:{}, requestId:{}", lockKey, currentValue, requestId);
            }
            if (DwRedis.del(lockKey) < 1) {
                DwLog.get().error("redis unlock fail:{}", lockKey);
            }
        } catch (Throwable e) {
            DwLog.get().error("redis unlock error:{}", lockKey);
        } finally {
            DwDistributedLockAdapter.getReentrantThreadLocal().get().remove(lockKey);
        }
    }
}

==> nest-infrastructure-middleware-redis-5.5.0.1000-20251217.112142-5-sources.jar: com/digiwin/dap/nest/infrastructure/middleware/redis/DwRedisLock.java <==
package com.digiwin.dap.nest.infrastructure.middleware.redis;

import com.digiwin.dap.nest.kernel.core.dapper.log.DwLog;
import com.digiwin.dap.nest.kernel.core.lock.distributed.DwDistributedLockAdapter;
import com.digiwin.dap.nest.kernel.core.lock.distributed.DwDistributedLockEntity;
import com.digiwin.dap.nest.kernel.core.lock.distributed.DwDistributedLockAspect;
import com.digiwin.dap.nest.kernel.core.util.DwStringUtil;
import com.digiwin.dap.nest.kernel.core.util.DwValidateUtil;
import com.digiwin.dap.nest.kernel.core.util.algorithm.id.DwUUID;
import com.digiwin.dap.nest.kernel.meta.adapter.JaI18nAdapter;
import com.digiwin.dap.nest.kernel.meta.function.JaFunctionTR;
import lombok.SneakyThrows;
import org.springframework.transaction.support.TransactionSynchronizationAdapter;
import org.springframework.transaction.support.TransactionSynchronizationManager;

import java.util.Set;

/**
 * redis 分布式锁
 *
 * @author chenjian
 * @since 2020年09月07日 10:00:27
 */
public class DwRedisLock {

    private DwRedisLock() {
    }


    /**
     * 使用demo 可参考{@link DwDistributedLockAspect}
     *
     * @author chenjian
     * @since 2020年10月13日 12:04:29
     */
    @SneakyThrows
    public static <T> T lock(DwDistributedLockEntity entity, JaFunctionTR<T> callable) {
        String lockKey = entity.getLockKey();
        String apiName = entity.getApiName();
        if (DwStringUtil.isEmpty(lockKey)) {
            DwLog.get().warn("lockKey is null,{}", apiName);
            return callable.apply();
        }
        // 判断是否可重入
        Set<String> reentrantLockKeySet = DwDistributedLockAdapter.getReentrantThreadLocal().getNotNull();
        if (reentrantLockKeySet.contains(lockKey)) {
            return callable.apply();
        }
        DwLog.biz("lockKey:{}", lockKey);
        // TODO 应对误删, 其实防不住, 待思考
        String requestId = lockKey + ":" + DwUUID.UUID19();
        boolean lock = DwDistributedLockAdapter.spin(entity, () -> DwRedis.setNx(lockKey, requestId, entity.getTimeout()));
        DwValidateUtil.isTrue(!lock, () -> JaI18nAdapter.getMessage(entity.getMessage()));
        try {
            reentrantLockKeySet.add(lockKey);
            return callable.apply();
        } finally {
            unlock(lockKey, requestId, entity.isTx());
        }
    }

    /**
     * 释放锁
     *
     * @author chenjian
     * @since 2020年08月11日 11:43:45
     */
    private static void unlock(String lockKey, String requestId, boolean tx) {
        if (TransactionSynchronizationManager.isSynchronizationActive() && tx) {
            try {
                TransactionSynchronizationManager.registerSynchronization(new TransactionSynchronizationAdapter() {
                    @Override
                    public void afterCompletion(int status) {
                        unlock(lockKey, requestId);
                    }
                });
            } catch (Throwable e) {
                DwLog.get().error("Transaction afterCompletion unLock lock failed", e);
                unlock(lockKey, requestId);
            }
        } else {
            unlock(lockKey, requestId);
        }
    }

    /**
     * TODO 暂用del
     * 是否要使用unlink
     * org.springframework.data.redis.core.RedisTemplate#unlink(java.lang.Object)
     */
    private static void unlock(String lockKey, String requestId) {
        try {
            String currentValue = DwRedis.getString(lockKey);
            if (!requestId.equals(currentValue)) {
                DwLog.get().warn("redis unlock warn {}:[currentValue:{}, requestId:{}", lockKey, currentValue, requestId);
            }
            if (DwRedis.del(lockKey) < 1) {
                DwLog.get().error("redis unlock fail:{}", lockKey);
            }
        } catch (Throwable e) {
            DwLog.get().error("redis unlock error:{}", lockKey);
        } finally {
            DwDistributedLockAdapter.getReentrantThreadLocal().get().remove(lockKey);
        }
    }
}
